#ifndef NWBINDRY_H
#define NWBINDRY_H

/****************************************/
/* Format of the LOGIN_CONTROL property */
/****************************************/

typedef struct {
       byte    accountexpirationdate[3];    /* byte 1 = yr, 2=month, 3=day */
       byte    accountdisabledflag;
       byte    passwordexpirationdate[3];   /* byte 1 = yr, 2=month, 3=day */
       byte    graceloginsremaining;
       nw_int  passwordexpirationinterval;
       byte    graceloginsallowed;
       byte    minimumpasswordlength;
       nw_int  maxcurrentconnections;
       byte    allowedlogintimebitmap[42];
       byte    lastlogindatetime[6];
       byte    restrictionsflag;            /* 0x01 - don't allow user to   */
                                            /*        change password       */ 
                                            /* 0x02 - unique passwords req. */
       byte    unused;
       nw_long maximumdiskusageinblocks;
       nw_int  badlogincount;
       nw_long nextresettime;
       byte    badloginnetworknumber[4];
       byte    badloginnodeaddress[6];
       byte    badloginsocket[2];
       } LOGINCONTROL_STRUCT;

/************************************************/
/* Format of SUPERVISORs USER_DEFAULTS property */
/************************************************/

typedef struct {
       byte    accountexpirationdate[3];   /* byte 1 = yr, 2=month, 3=day */
       byte    requireuniquepasswords;     /* 0x02 = unique passwords     */
                                           /*        required             */
       nw_int  passwordexpirationinterval;
       byte    graceloginsallowed;
       byte    minimumpasswordlength;
       nw_int  maxcurrentconnections;
       byte    allowedlogintimebitmap[42];
       nw_long accountbalance;
       nw_long lowbalance;                 /* unlimited = 0x80000000 */
       nw_long maximumdiskusageinblocks;   /* no limit = 0x7fffffff  */
       byte    createhomedirectory;        /* 0x00 = No, 0x01 = yes  */
       } USERDEFAULTS_STRUCT;

/*******************************/
/* Bindery Services prototypes */
/*******************************/

#ifdef __cplusplus
extern "C" {
#endif
int   AddBinderyObjectToSet(int objectType,
                 char *objectName,
                 char *propertyName,int memberType,
                 char *memberName);
int   ChangeBinderyObjectPassword(int objectType,
                 char *objectName,char *oldPassword,
                 char *newPassword);
int   ChangeBinderyObjectSecurity(byte newSecurity,int objectType,
                 char *objectName);
int   ChangePropertySecurity(int objectType,
                 char *objectName,
                 byte newPropSecurity,char *propName);
int   CloseBindery(void);
int   CreateBinderyObject(byte flag,byte security,int objectType,
                 char *objectName);
int   CreateProperty(int objectType,char *objectName,
                 byte propFlags,byte propSecurity,char *prop_name);
int   DeleteBinderyObject(int objectType,
                 char *objectName);
int   DeleteBinderyObjectFromSet(int objectType,
                 char *objectName,char *propertyName,
                 int memberType,char *member_name);
int   DeleteProperty(int objectType,char *objectName,
                 char *propName);
int   GetBinderyAccessLevel(long *objectID,
                 byte *accessLevel);
int   GetBinderyObjectID(int objectType,
                 char *objectName,long *objectID);
int   GetBinderyObjectName(long objectID,
                 int *objectType,char *objectName);
int   IsBinderyObjectInSet(int objectType,
                 char *objectName,char *propertyName,
                 int memberType,char *member_name);
int   OpenBindery(void);
int   ReadPropertyValue(int objectType,char *objectName,
                 char *propertyName,int segment,
                 char *propertyValues,
                 byte *moreSegments,byte *propertyFlag);
int   RenameBinderyObject(int objectType,
                 char *objectName,char *newObjectName);
int   ScanBinderyObject(int scanObjectType,
                 char *scanObjectName,
                 long *lastObjectID,
                 int *objectType,char *objectName,
                 byte *objectHasProperties,
                 byte *objectSecurity,
                 byte *objectFlag);
int   ScanProperty(int objectType,char *objectName,
                 char *scanPropertyName,
                 long *lastSequence,
                 char *propertyName,byte *propertyFlags,
                 byte *propertySecurity,
                 byte *propertyHasValue,
                 byte *moreProperties);
int   VerifyBinderyObjectPassword(int objectType,
                 char *objectName,char *password);
int   WritePropertyValue(int objectType,
                 char *objectName,int segment,byte erase_remaining,
                 char *propName,byte *value);
#ifdef __cplusplus
}
#endif

#endif
