#ifndef NWQUEUE_H
#define NWQUEUE_H

/***************************/
/* QMS Job Entry structure */
/***************************/

typedef struct {
   byte client_station;
   byte client_task_number;
   long client_id_number;
   long target_server_id_number;   /* -1 = non specified */
   byte target_execution_time[6];  /* year,month,day,hours,minutes,seconds */
   byte job_entry_time[6];         /* year,month,day,hours,minutes,seconds */
   word job_number;
   word job_type;
   byte job_position;
   byte job_control_flags;
   byte job_file_name[14];
   byte job_file_handle[6];
   byte server_station;
   byte server_task_number;
   long server_id_number;
   char text_job_description[50];
   byte client_record_area[152];
   } QUEUE_JOB_ENTRY;

/*************************/
/* QMS job control flags */
/*************************/

#define QF_OPERATOR_HOLD  0x80
#define QF_USER_HOLD      0x40
#define QF_ENTRY_OPEN     0x20
#define QF_ENTRY_RESTART  0x10
#define QF_AUTO_START     0x08

/**********************************/
/* print queue client record area */
/**********************************/

typedef struct {
   byte   version;
   byte   tab_size;
   nw_int copies;
   word   control_flags;
   int    maximum_lines;
   int    maximum_characters;
   char   form_name[16];
   byte   reserved[6];
   char   banner_name[13];
   char   fname_on_banner[13];
   char   fname_in_header[14];
   char   directory_path[80];
   } PSRV_CLIENT_AREA;

/*****************************/
/* print queue control flags */
/*****************************/

#define PSRVF_TEXT        0x4000
#define PSRVF_BANNER      0x8000
#define PSRVF_NOFF        0x0800

/*****************************/
/* Queue Services prototypes */
/*****************************/

#ifdef __cplusplus
extern "C" {
#endif
int   AbortServicingQueueJobAndFile(long queueID,word jobNumber,
                 int fileHandle);
int   AttachQueueServerToQueue(long queueID);
int   ChangeQueueJobEntry(long queueID,
                 QUEUE_JOB_ENTRY *jobEntry);
int   ChangeQueueJobPosition(long queueID,word jobNumber,
                 byte newPosition);
int   ChangeToClientRights(long queueID,word jobNumber);
int   CloseFileAndAbortQueueJob(long queueID,word jobNumber,
                 int fileHandle);
int   CloseFileAndStartQueueJob(long queueID,word jobNumber,
                 int fileHandle);
int   CreateQueue(char *queueName,int queueType,
                 char directoryHandle,char *pathName,
                 long *queueID);
int   CreateQueueJobAndFile(long queueID,
                 QUEUE_JOB_ENTRY *jobEntry,
                 int *fileHandle);
int   DestroyQueue(long queueID);
int   DetachQueueServerFromQueue(long queueID);
int   FinishServicingQueueJobAndFile(long queueID,
                 word jobNumber,long charge,int fileHandle);
int   GetQueueJobList(long queueID,word *jobCount,
                 word *jobNumbers );
int   GetQueueJobsFileSize(long queueID,word jobNumber,
                 long *fileSize);
int   ReadQueueCurrentStatus(long queueID,
                 byte *queueStatus,byte *numberOfJobs,
                 byte *numberOfServers,
                 long *serverIDList,
                 word *serverStationList);
int   ReadQueueJobEntry(long queueID,word jobNumber,
                 QUEUE_JOB_ENTRY *jobEntry);
int   ReadQueueServerCurrentStatus(long queueID,long serverID,
                 byte serverStation,byte *serverStatusRecord);
int   RemoveJobFromQueue(long queueID,word jobNumber);
int   RestoreQueueServerRights(void);
int   ServiceQueueJobAndOpenFile(long queueID,word targetJobType,
                 QUEUE_JOB_ENTRY *jobEntry,
                 int *fileHandle);
int   SetQueueCurrentStatus(long queueID,byte queueStatus);
int   SetQueueServerCurrentStatus(long queueID,
                 byte *serverStatusRecord);
#ifdef __cplusplus
}
#endif
     
#endif
