#ifndef NWSERVER_H
#define NWSERVER_H

/***********************************/
/* File Server Services prototypes */
/***********************************/

typedef struct {
   long system_elapsed_time;
   int cache_buffer_count;
   int cache_buffer_size;
   int dirty_cache_buffers;
   long cache_read_requests;
   long cache_write_requests;
   long cache_hits;
   long cache_misses;
   long physical_read_requests;
   long physical_write_requests;
   int physical_read_errors;
   int physical_write_errors;
   long cache_get_requests;
   long cache_full_write_requests;
   long cache_partial_write_requests;
   long background_dirty_writes;
   long background_aged_writes;
   long total_cache_writes;
   long cache_allocations;
   int thrashing_count;
   int LRU_block_was_dirty;
   int read_beyond_write;
   int fragmented_write_occurred;
   int cache_hit_unavail_block;
   int cache_block_scrapped;
   } DISK_CACHE_STATISTICS;         /* GetDiskCacheStatistics */

typedef struct {
   char server_name[48];
   byte netware_version;
   byte netware_subversion;
   int connections_supported;
   int connections_in_use;
   int max_connected_volumes;
   byte os_revision;
   byte SFT_level;
   byte TTS_level;
   int peak_connections_used;
   byte accounting_version;
   byte VAP_version;
   byte queuing_version;
   byte print_server_version;
   byte virtual_console_version;
   byte security_restrictions_level;
   byte internet_bridge_version;
   byte reserved[60];
   } FILE_SERVER_INFO;		  /* GetFileServerInformation */

typedef struct {
   word  task_number;
   dword parent_dir_entry;
   dword directory_entry;
   byte  lock_flag;
   byte  access_flag;
   byte  lock_type;
   byte  name_space;
   byte  volume_number;
   byte  reserved;
   char  file_name[14];
   } OPEN_FILES_INFO;               /* GetConnectionsOpenFiles */

typedef struct {
   word buffer_length;
   long system_elapsed_time;
   byte physical_disk_channel;
   byte drive_removable_flag;
   byte physical_drive_type;
   byte controller_drive_number;
   byte controller_number;
   byte controller_type;
   long drive_size;
   int drive_cylinders;
   byte drive_heads;
   byte sectors_per_track;
   char drive_definition_string[64];
   int io_error_count;
   long hot_fix_table_start;
   int hot_fix_table_size;
   int hot_fix_blocks_available;
   byte hot_fix_disabled;
   } PHYSICAL_DISK_STATISTICS;         /* GetPhysicalDiskStatistics */

#ifdef __cplusplus
extern "C" {
#endif
int   CheckConsolePrivileges(void);
int   ClearConnectionNumber(int connection);
int   ConvertPathToDirectoryEntry(byte directoryHandle,
                 char *directoryPath,word *volumeNumber,
                 dword *dirEntryNumber);
int   DisableFileServerLogin(void);
int   DisableTransactionTracking(void);
int   DownFileServer(int forceIt);
int   EnableFileServerLogin(void);
int   EnableTransactionTracking(void);
int   GetBinderyObjectDiskSpaceLeft(long objectID,
                 long *systemElapsedTime,
                 long *unusedDiskBlocks,
                 byte *restrictionsEnforced);
int   GetConnectionsOpenFiles(int connection,int maxEntries,
                 OPEN_FILES_INFO *openFiles,
                 int *fileCount);
int   GetConnectionsUsageStatistics(int connection,
                 long *systemElapsedTime,
                 double *bytesRead,
                 double *bytesWritten,
                 long *totalRequestPackets);
int   GetDiskCacheStatistics(
                 DISK_CACHE_STATISTICS *buffer);
int   GetDiskUtilisation(long objectID,byte volumeNumber,
                 word *usedDirectories,word *usedFiles,
                 word *usedBlocks);
void  GetFileServerDateTime(nw_time *time,
                 nw_date *date,int *dayOfWeek);
int   GetFileServerInformation(
                 FILE_SERVER_INFO *info);
int   GetFileServerLoginStatus(int *loginEnabled);
int   GetNetworkSerialNumber(dword *serverSerialNumber,
                 word  *appSerialNumber);
int   GetPathFromDirectoryEntry(byte nameSpace,byte volumeNumber,
                 dword directoryEntry,char *path);
int   GetPhysicalDiskStatistics(byte physicalDiskNumber,
                 PHYSICAL_DISK_STATISTICS *diskStats);
int   GetSemaphoreInformation(char *semaphoreName,
                 int maxConnections,word *openCount,
                 int *semaphoreValue,
                 word *logicalConnections,
                 word *taskNumbers,
                 int *connectionCount );
int   SendConsoleBroadcast(byte connectionCount,
                 byte *connections,char *message);
#ifdef __cplusplus
}
#endif

#endif
